#define __16f690
#include "pic/pic16f690.h"

/****************/

typedef unsigned int word;
typedef unsigned char byte;

#define OUT 0
#define IN  1

#define LOW  0
#define HIGH 1

/***************/
 


/* word at 0x2007 __CONFIG = _FCMEN_ON & _WDT_OFF; */
word at 0x2007 __CONFIG = _INTRC_OSC_NOCLKOUT & _WDT_OFF & _PWRTE_OFF & _MCLRE_OFF & _CP_OFF & _BOR_OFF & _IESO_OFF & _FCMEN_OFF;


#define  L_MOTOR_A_DIR  TRISC6
#define  L_MOTOR_B_DIR  TRISC7
#define  R_MOTOR_A_DIR  TRISB4
#define  R_MOTOR_B_DIR  TRISB6

#define  L_SWITCH_DIR   TRISC1
#define  R_SWITCH_DIR   TRISC2

#define  L_MOTOR_A      RC6
#define  L_MOTOR_B      RC7
#define  R_MOTOR_A      RB4
#define  R_MOTOR_B      RB6

#define  L_SWITCH       RC1
#define  R_SWITCH       RC2


#define  SERIAL_RX_DIR  TRISB5
#define  SERIAL_RX      RB5



/* {{{ void init() */
void init()
{
   /* disable comparators */
   CM1CON0 = 0;
   CM2CON0 = 0;
   CM2CON1 = 0;

   /* disable analog inputs */
   ANSEL = 0;
   ANSELH = 0;

   L_MOTOR_A_DIR = OUT;
   L_MOTOR_B_DIR = OUT;
   R_MOTOR_A_DIR = OUT;
   R_MOTOR_B_DIR = OUT;

   L_SWITCH_DIR  = IN;
   R_SWITCH_DIR  = IN;

   SERIAL_RX_DIR = IN;

   /* init serial baud rate generator (2400 baud) */
   BRGH   =  0;
   BRG16  =  0;
   SPBRGH =  0;
   SPBRG  = (unsigned char) 25;

   /* init serial rx/tx */
   SYNC = 0;
   SPEN = 1;
   CREN = 1;
   TXEN = 1;
} /* void init() */


/* }}} */

/* {{{ void delay_ms(long ms) */
void delay_ms(long ms)
{
   long i;

   while(ms--)
   { for(i = 0; i < 45; i++); }
} /* void delay_ms(long ms) */


/* }}} */
/* {{{ byte serial_getc() */
byte serial_getc()
{
   byte framing_error = 0;
   byte overflow_error = 0;
   byte input_byte = 0;


   //while(!RCIF);

   if(RCIF)
   {
     // get error flags from RCSTA
      framing_error  = FERR;
      overflow_error = OERR;

      // read RCREG to get data
      input_byte = RCREG;


      if(framing_error)
      { input_byte = 0; }

      if(overflow_error)
      {
         /* clear overrun flag */
         CREN = 0;
         CREN = 1;
      }
   }

   return(input_byte);
} /* byte serial_getc() */


/* }}} */
/* {{{ byte serial_putc() */
void serial_putc(byte output_byte)
{
   TXREG = output_byte; // Add a character to the output buffer
   while(!TXIF);  // Wait while the output buffer is full
} /* byte serial_putc() */


/* }}} */

/* movement functions */
/* {{{ void go_forward() */
void go_forward()
{
   L_MOTOR_A = HIGH;
   L_MOTOR_B = LOW;
   R_MOTOR_A = HIGH;
   R_MOTOR_B = LOW;
} /* void go_forward() */


/* }}} */
/* {{{ void go_reverse() */
void go_reverse()
{
   L_MOTOR_A = LOW;
   L_MOTOR_B = HIGH;
   R_MOTOR_A = LOW;
   R_MOTOR_B = HIGH;
} /* void go_reverse() */


/* }}} */
/* {{{ void stop() */
void stop()
{
   L_MOTOR_A = LOW;
   L_MOTOR_B = LOW;
   R_MOTOR_A = LOW;
   R_MOTOR_B = LOW;
} /* void stop() */


/* }}} */
/* {{{ void rotate_left() */
void rotate_left()
{
   L_MOTOR_A = HIGH;
   L_MOTOR_B = LOW;
   R_MOTOR_A = LOW;
   R_MOTOR_B = HIGH;
} /* void rotate_left() */


/* }}} */
/* {{{ void rotate_right() */
void rotate_right()
{
   L_MOTOR_A = LOW;
   L_MOTOR_B = HIGH;
   R_MOTOR_A = HIGH;
   R_MOTOR_B = LOW;
} /* void rotate_right() */


/* }}} */


#define RAND_MAX 32767
static unsigned long int next = 1;
/* {{{ int rand(void) */
int rand(void)
{
    next = next * 1103515245UL + 12345;
    return (unsigned int)(next/65536) % (RAND_MAX + 1U);
} /* int rand(void) */


/* }}} */
/* {{{ void srand(unsigned int seed) */
void srand(unsigned int seed)
{ next = seed; }


/* }}} */


// {{{ void main()
void main()
{
   byte input_char = 0;

   init();

   stop();

   while(1)
   {
      if(L_SWITCH == HIGH)
      {
         go_reverse();
         delay_ms(250);
         rotate_right();
         delay_ms(250 + (rand() % 250));
      }
      else if(R_SWITCH == HIGH)
      {
         go_reverse();
         delay_ms(250);
         rotate_left();
         delay_ms(250 + (rand() % 250));
      }
      else
      {
         go_forward();
      } // if(L_SWITCH == HIGH)


      /*
      input_char = serial_getc();

      switch(input_char)
      {
         case 'F': go_forward();   break;
         case 'R': rotate_right(); break;
         case 'L': rotate_left();  break;
         case 'B': go_reverse();   break;
         case 'S': stop();         break;
         default:
            break;
      } // switch(input_char)

      delay_ms(200);
      */
   } // while(1)
} // void main()


// }}}
